/*
imatrix.c

matrix manipulation library
functions on matrices
version 0.03
Peter A. Cejchan, GPL
Tue Mar 30 12:16:31 GMT+1 2004 <cej@gli.cas.cz | pac7@post.cz | pac@plan9.cz>



*/


#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>
#include "libmatrix.h" 
#include "imatrix.h" 


// functions:

iMatrix *
imat_alloc(int rows, int cols){
// allocates memory for a matrix

	int i,j;
	iMatrix *M;

	M=(iMatrix*)malloc(sizeof(iMatrix));
	M->rows=rows; M->cols=cols;
	M->x=(int**)malloc(M->rows*sizeof(int*));
	for(j=0;j<rows;j++) M->x[j]=(int*)malloc(cols*sizeof(int));
	for(i=0;i<M->rows;i++)
		for(j=0;j<M->cols;j++) M->x[i][j]=0.0;
	return M;
}


int
imat_read(iMatrix *M){
// reads in a whitespace and newline separated matrix

	char *buf, *av[MAXCOLS];
	int i, j, n;
	for(i=0; i<M->rows; i++){
		if((buf = Brdstr(&bin, '\n', 1)) != nil){	
			// skip comment lines
			while((buf[0]=='#') || (buf[0]=='>')){

				print("comments\n");	
				if((buf = Brdstr(&bin, '\n', 1)) == nil) return 0;	// read another line
			}	
			n=tokenize(buf, av, M->cols);
			// skip whitespace lines
			while(n==0){
				print("whitespace\n");	
				if((buf = Brdstr(&bin, '\n', 1)) == nil) return 0;	// read another line
				n=tokenize(buf, av, M->cols);
			}	
			for(j=0; j<M->cols; j++)
				M->x[i][j]=atoi(av[j]);
			free(buf);
		}
		else return 0;
	}
	return 1;
}


void 
imat_write(iMatrix *M){
//writes out a matrix 
	int i, j;
	print("%d %d ", M->rows, M->cols);
	for(i=0; i<M->rows; i++){
		for(j=0; j<M->cols; j++){
			print("%d ", M->x[i][j]);
		}
		print("\n");
	}
	return;
}


int 
imat_max(iMatrix *M){
//prints out row and col index of first occurrence of max value
	int i, j, maxi, maxj;
	int maxx;
	maxx = -MAXINT;
	for(i=0; i<M->rows; i++){
		for(j=0; j<M->cols; j++){
			if(M->x[i][j] > maxx) {
				maxx = M->x[i][j];
				maxi = i; maxj = j;
			}
		}
	}
	print("%d %d\n", maxi, maxj);
	return 1;
}

int 
imat_transpose(iMatrix *M){
// matrix transposition

	iMatrix tmp;
	int i, j;

	for(i=0; i<M->rows; i++){
		for(j=0; j<M->cols; j++){
			tmp.x[j][i] = M->x[i][j];
		}
	}
	for(i=0; i<M->rows; i++){
		for(j=0; j<M->cols; j++){
			M->x[i][j] = tmp.x[i][j];
		}
	}
	return 1;
}



void 
imat_ident(iMatrix *M)
// store identity matrix in M
{
	int i, j;
	for(i=0; i<M->rows; i++)
		for(j=0; j<M->cols; j++)
			if(i==j)
				M->x[j][i] = 1;
			else
				M->x[j][i] = 0;
}

void 
imat_add(iMatrix *a, iMatrix *b, iMatrix *out)
// matrix addition
{	int	i, j;
	for ( i=0; i<a->rows; i++ )
		for ( j=0; j<a->cols; j++ )
			out->x[i][j] = a->x[i][j]+b->x[i][j];
}


void 
imat_sub(iMatrix *a, iMatrix *b, iMatrix *out)
// matrix subtraction

{
	int	i, j;
	for ( i=0; i<a->rows; i++ )
		for ( j=0; j<a->cols; j++ )
			out->x[i][j] = a->x[i][j]-b->x[i][j];
}

void 
imat_mult(iMatrix *a, iMatrix *b, iMatrix *out)
// matrix multiplication
{
	int	i, j, k;
	double sum;
	for ( i=0; i<a->rows; i++ )
		for  ( j=0; j<b->cols; j++ )
		{
			sum = 0.0;
			for ( k=0; k<a->cols; k++ )
				sum += a->x[i][k]*b->x[k][j];
			out->x[i][j] = sum;
		}
}
