/*
matrix.c

matrix manipulation library
functions on matrices
version 0.03
Peter A. Cejchan, GPL
Mon Mar 29 12:39:45 GMT+1 2004 <cej@gli.cas.cz | pac7@post.cz | pac@plan9.cz>



*/


#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>
#include "libmatrix.h" 
#include "matrix.h" 

// functions:

int 
mat_readheader(int *rows, int *cols)
// reads #rows #columns
{
	char *buf, *av[MAXCOLS], first;
	int i, j, n;

	n=0;
	while(n==0){
		if((buf = Brdstr(&bin, '\n', 1)) == nil) return 0;			// nothing to read
		first=buf[0];
//				print("first==%c\n", first);
		n=tokenize(buf, av, 2);
		if(n!=0){
			// skip comment lines
			if((first=='#') || (first=='>')){
//				print("comments\n");
				n=0;
			}
		}
		else{
		// skip whitespace lines
//			print("whitespace\n");	
		}
	}	// ends up with n > 0
	*rows=atoi(av[0]);
	*cols=atoi(av[1]);
	return 0;
}


Matrix *
mat_alloc(int rows, int cols)
// allocates memory for a matrix
{
	int i,j;
	Matrix *M;

	M=(Matrix*)malloc(sizeof(Matrix));
	M->rows=rows; M->cols=cols;
	M->x=(double**)malloc(M->rows*sizeof(double*));
	for(j=0;j<rows;j++) M->x[j]=(double*)malloc(cols*sizeof(double));
	for(i=0;i<M->rows;i++)
		for(j=0;j<M->cols;j++) M->x[i][j]=0.0;
	return M;
}


int
mat_read(Matrix *M)
// reads in a whitespace and newline separated matrix
{
	char *buf, *av[MAXCOLS];
	int i, j, n;
	for(i=0; i<M->rows; i++){
		if((buf = Brdstr(&bin, '\n', 1)) != nil){	
			// skip comment lines
			while((buf[0]=='#') || (buf[0]=='>')){

				print("comments\n");	
				if((buf = Brdstr(&bin, '\n', 1)) == nil) return 0;	// read another line
			}	
			n=tokenize(buf, av, M->cols);
			// skip whitespace lines
			while(n==0){
				print("whitespace\n");	
				if((buf = Brdstr(&bin, '\n', 1)) == nil) return 0;	// read another line
				n=tokenize(buf, av, M->cols);
			}	
			for(j=0; j<M->cols; j++)
				M->x[i][j]=atof(av[j]);
			free(buf);
		}
		else return 0;
	}
	return 1;
}


void 
mat_write(Matrix *M)
//writes out a matrix
{
	int i, j;
	print("%d %d ", M->rows, M->cols);
	for(i=0; i<M->rows; i++){
		for(j=0; j<M->cols; j++){
			print("%f ", M->x[i][j]);
		}
		print("\n");
	}
	return;
}

int 
mat_max(Matrix *M)
//prints out row and col index of first occurrence of max value
{
	int i, j, maxi, maxj;
	double maxx;
	maxx = -MAXFLOAT;
	for(i=0; i<M->rows; i++){
		for(j=0; j<M->cols; j++){
//			print("%f ", M->x[i][j]);
			if(M->x[i][j] > maxx) {
				maxx = M->x[i][j];
				maxi = i; maxj = j;
			}
		}
//		print("\n");
	}
//	print("%f\n", maxx);
	print("%d %d\n", maxi, maxj);
	return 1;
}

int 
mat_transpose(Matrix *M)
// matrix transposition
{
	Matrix tmp;
	int i, j;

	for(i=0; i<M->rows; i++){
		for(j=0; j<M->cols; j++){
			tmp.x[j][i] = M->x[i][j];
		}
	}
	for(i=0; i<M->rows; i++){
		for(j=0; j<M->cols; j++){
			M->x[i][j] = tmp.x[i][j];
		}
	}
	return 1;
}


void 
mat_ident(Matrix *M)
// store identity matrix in M
{
	int i, j;
	for(i=0; i<M->rows; i++){
		for(j=0; j<M->cols; j++){
			if(i==j){
				M->x[j][i] = 1;
			}
			else{
				M->x[j][i] = 0;
			}
		}
	}
}

void 
mat_add(Matrix *a, Matrix *b, Matrix *out)
// matrix addition
{
	int	i, j;
	for ( i=0; i<a->rows; i++ )
		for ( j=0; j<a->cols; j++ )
			out->x[i][j] = a->x[i][j]+b->x[i][j];

}


void 
mat_sub(Matrix *a, Matrix *b, Matrix *out)
// matrix subtraction

{
	int	i, j;
	for ( i=0; i<a->rows; i++ )
		for ( j=0; j<a->cols; j++ )
			out->x[i][j] = a->x[i][j]-b->x[i][j];
}

void 
mat_mult(Matrix *a, Matrix *b, Matrix *out)
// matrix multiplication
{
	int	i, j, k;
	double sum;
	for ( i=0; i<a->rows; i++ )
		for  ( j=0; j<b->cols; j++ )
		{
			sum = 0.0;
			for ( k=0; k<a->cols; k++ )
				sum += a->x[i][k]*b->x[k][j];
			out->x[i][j] = sum;
		}
}

